@@ -1,15 +1,18 @@
 package KinoSearch::Analysis::Tokenizer;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Analysis::Analyzer );
 
+use KinoSearch::Analysis::TokenBatch;
+
 our %instance_vars = __PACKAGE__->init_instance_vars(
 
     # constructor params / members
     token_re => undef,    # regex for a single token
 
     # members
-    separator_re  => undef,    # regex for separations between tokens
-    tokenizing_re => undef,    # regex optimized for lexing
+    separator_re => undef,    # regex for separations between tokens
 );
 
 sub init_instance {
@@ -17,7 +20,7 @@ sub init_instance {
 
     # supply defaults if token_re wasn't specified
     if ( !defined $self->{token_re} ) {
-        $self->{token_re}     = qr/\w+(?:'\w+)?/;
+        $self->{token_re}     = qr/\b\w+(?:'\w+)?\b/;
         $self->{separator_re} = qr/\W*/;
     }
 
@@ -31,59 +34,36 @@ sub init_instance {
                 $self->{token_re}  # a token, 
                 |\z                # or the end of the string
             )/xsm;
-        $self->{tokenizing_re} = qr/
-            ($self->{token_re})           # capture token to $1
-            (.*?                          # capture separator to $2
-                (?=
-                    $self->{token_re}
-                    |\z
-                )
-            )/xsm;
-    }
-
-    # if internally defined token_re and separator_re...
-    if ( !defined $self->{tokenizing_re} ) {
-        # use slightly more efficient non-lookahead tokenizing_re scheme
-        $self->{tokenizing_re} = qr/
-            ($self->{token_re})           # capture token to $1
-            ($self->{separator_re})       # capture separator to $2
-            /xsm;
     }
 }
 
 sub analyze {
-    my ( $self, $field ) = @_;
-    my $tokenizing_re = $self->{tokenizing_re};
-    my ( @tokens, @start_offsets, @end_offsets );
+    my ( $self, $token_batch ) = @_;
 
-    # use the first element in the terms array as input
-    my $terms = $field->get_terms;
+    my $new_token_batch = KinoSearch::Analysis::TokenBatch->new;
+    my $token_re        = $self->{token_re};
+    my $separator_re    = $self->{separator_re};
 
     # alias input to $_
-    for ( $terms->[0] ) {
+    while ( $token_batch->next ) {
+        local $_ = $token_batch->get_text;
 
-        # if the field content is an empty string, bail out
-        return unless bytes::length($_);
-
-        # set the pos for input to the start of the first token
+        # ensure that pos is set to 0 for this scalar
         pos = 0;
-        m/$self->{separator_re}/g;
 
-        my $last_pos = 0;
-        push @start_offsets, pos;
+        # accumulate token start_offsets and end_offsets
+        my ( @starts, @ends );
+        1 while ( m/$separator_re/g and push @starts,
+            pos and m/$token_re/g and push @ends, pos );
 
-        # lex through input, capturing tokens and recording positional data
-        while (m/$tokenizing_re/g) {
-            push @tokens,        $1;
-            push @start_offsets, pos;
-            push @end_offsets,   pos() - bytes::length($2);
-        }
+        # correct for overshoot
+        $#starts = $#ends;
 
-        # the last loop will always record 1 extra start, so truncate
-        $#start_offsets = $#tokens;
+        # add the new tokens to the batch
+        $new_token_batch->add_many_tokens( $_, \@starts, \@ends );
     }
 
-    $field->set_tokenbatch( \@tokens, \@start_offsets, \@end_offsets );
+    return $new_token_batch;
 }
 
 1;
@@ -134,30 +114,32 @@ based on the value of C<token_re>.
     # tokenized by $word_char_tokenizer
     @tokens = qw( Eats Shoots and Leaves   );
 
-=head1 CONSTRUCTOR
+=head1 METHODS
 
 =head2 new
-
-    my $tokenizer = KinoSearch::Analysis::Tokenizer->new(
-        token_re => $matches_one_token, );
-
-Construct a Tokenizer object.  
-
-B<token_re> must be a pre-compiled regular expression matching one token.  It
-must not use any capturing parentheses, though non-capturing parentheses are
-fine:
-
+    
     # match "O'Henry" as well as "Henry" and "it's" as well as "it"
     my $token_re = qr/
+            \b        # start with a word boundary
             \w+       # Match word chars.
             (?:       # Group, but don't capture...
                '\w+   # ... an apostrophe plus word chars.
             )?        # Matching the apostrophe group is optional.
+            \b        # end with a word boundary
         /xsm;
-    my $apostrophizing_tokenizer
-        = KinoSearch::Analysis::Tokenizer->new( token_re => $token_re, );
+    my $tokenizer = KinoSearch::Analysis::Tokenizer->new(
+        token_re => $token_re, # default: what you see above
+    );
 
-Incidentally, the above token_re is the default value.
+Constructor.  Takes one hash style parameter.
+
+=over
+
+=item *
+
+B<token_re> - must be a pre-compiled regular expression matching one token.
+
+=back
 
 =head1 COPYRIGHT
 
@@ -165,6 +147,6 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =cut
