@@ -1,4 +1,6 @@
 package KinoSearch::Index::FieldsWriter;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Util::Class );
 
@@ -42,13 +44,45 @@ sub add_doc {
         push @to_write, $_->get_compressed
             ? compress( $_->get_value )
             : $_->get_value;
+        push @to_write, $_->get_tv_string;
     }
 
     # write out data
-    my $lu_template = 'V' . ( 'VaT' x scalar @stored );
+    my $lu_template = 'V' . ( 'VaTT' x scalar @stored );
     $self->{fdata_stream}->lu_write( $lu_template, @to_write );
 }
 
+sub add_segment {
+    my ( $self, $seg_reader, $doc_map, $field_num_map ) = @_;
+    my ( $findex_stream, $fdata_stream )
+        = @{$self}{qw( findex_stream fdata_stream )};
+    my $fields_reader = $seg_reader->get_fields_reader;
+
+    my $max = $seg_reader->max_doc;
+    return unless $max;
+    $max -= 1;
+    for my $orig ( 0 .. $max ) {
+        # if the doc isn't deleted, copy it to the new seg
+        next unless defined $doc_map->get($orig);
+
+        # write pointer
+        $findex_stream->lu_write( 'Q', $fdata_stream->tell );
+
+        # retrieve all fields
+        my ( $num_fields, $all_data ) = $fields_reader->fetch_raw($orig);
+
+        # write number of fields
+        $fdata_stream->lu_write( 'V', $num_fields );
+
+        # write data for each field
+        for ( 1 .. $num_fields ) {
+            my ( $field_num, @some_data ) = splice( @$all_data, 0, 4 );
+            $fdata_stream->lu_write( 'VaTT', $field_num_map->get($field_num),
+                @some_data );
+        }
+    }
+}
+
 sub finish {
     my $self = shift;
     $self->{fdata_stream}->close;
@@ -76,7 +110,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
