@@ -1,4 +1,6 @@
 package KinoSearch::Index::SegReader;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Index::IndexReader );
 
@@ -26,7 +28,16 @@ our %instance_vars = __PACKAGE__->init_instance_vars(
     norms_readers    => {},
 );
 
-__PACKAGE__->ready_get(qw( freq_stream prox_stream deldocs ));
+__PACKAGE__->ready_get(
+    qw(
+        finfos
+        fields_reader
+        freq_stream
+        prox_stream
+        deldocs
+        seg_name
+        )
+);
 
 # use KinoSearch::Util::Class's new()
 # Note: can't inherit IndexReader's new() without recursion problems
@@ -42,7 +53,7 @@ sub init_instance {
         seg_name => $seg_name,
     );
     $self->{deldocs}->read_deldocs( $invindex, "$seg_name.del" )
-        if ( $self->has_deletions );
+        if ( $invindex->file_exists("$seg_name.del") );
 
     # initialize a CompoundFileReader
     my $comp_file_reader = $self->{comp_file_reader}
@@ -79,11 +90,27 @@ sub init_instance {
 
 sub max_doc { shift->{fields_reader}->get_size }
 
-sub has_deletions {
+sub num_docs {
     my $self = shift;
-    $self->{invindex}->file_exists("$self->{seg_name}.del");
+    return $self->max_doc - $self->{deldocs}->get_num_deletions;
 }
 
+sub delete_docs_by_term {
+    my ( $self, $term ) = @_;
+    my $term_docs = $self->term_docs($term);
+    $self->{deldocs}->delete_by_term_docs($term_docs);
+}
+
+sub commit_deletions {
+    my $self = shift;
+    return unless $self->{deldocs}->get_num_deletions;
+    my $filename = $self->{seg_name} . ".del";
+    $self->{deldocs}
+        ->write_deldocs( $self->{invindex}, $filename, $self->max_doc );
+}
+
+sub has_deletions { shift->{deldocs}->get_num_deletions }
+
 sub _open_norms {
     my $self = shift;
     my ( $seg_name, $finfos, $comp_file_reader )
@@ -91,11 +118,10 @@ sub _open_norms {
     my $max_doc = $self->max_doc;
 
     # create a NormsReader for each indexed field.
-    for my $orig_num ( 0 .. $finfos->size - 1 ) {
-        my $finfo = $finfos->info_by_orig_num($orig_num);
+    for my $finfo ( $finfos->get_infos ) {
         next unless $finfo->get_indexed;
-        my $instream
-            = $comp_file_reader->open_instream("$seg_name.f$orig_num");
+        my $filename = "$seg_name.f" . $finfo->get_field_num;
+        my $instream = $comp_file_reader->open_instream($filename);
         $self->{norms_readers}{ $finfo->get_name }
             = KinoSearch::Index::NormsReader->new(
             instream => $instream,
@@ -129,6 +155,7 @@ sub term_docs {
 
 sub norms_reader {
     my ( $self, $field_name ) = @_;
+    return unless exists $self->{norms_readers}{$field_name};
     return $self->{norms_readers}{$field_name};
 }
 
@@ -141,10 +168,23 @@ sub get_field_names {
     return \@names;
 }
 
+sub generate_field_infos {
+    my $self       = shift;
+    my $new_finfos = $self->{finfos}->clone;
+    $new_finfos->set_from_file(0);
+    return $new_finfos;
+}
+
 sub fetch_doc {
     $_[0]->{fields_reader}->fetch_doc( $_[1] );
 }
 
+sub segreaders_to_merge {
+    my ( $self, $all ) = @_;
+    return $self if $all;
+    return;
+}
+
 1;
 
 __END__
@@ -165,7 +205,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
