@@ -1,7 +1,11 @@
 package KinoSearch::Search::TermScorer;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( KinoSearch::Search::Scorer );
 
+use KinoSearch::Search::Scorer qw( %score_batch_args );
+
 our %instance_vars = __PACKAGE__->init_instance_vars(
     # constructor params
     weight       => undef,
@@ -11,11 +15,11 @@ our %instance_vars = __PACKAGE__->init_instance_vars(
 
 sub new {
     my $self = shift->SUPER::new;
-    verify_args(\%instance_vars, @_);
-    my %args = (%instance_vars, @_);
+    verify_args( \%instance_vars, @_ );
+    my %args = ( %instance_vars, @_ );
 
     $self->_init_child;
-    
+
     $self->_set_term_docs( $args{term_docs} );
     $self->_set_norms( $args{norms_reader}->get_bytes );
     $self->set_similarity( $args{similarity} );
@@ -27,11 +31,6 @@ sub new {
     return $self;
 }
 
-sub do_score_batch {
-    my ( $self, %args ) = @_;
-    _do_score_batch( $self, @args{qw( start end hit_collector )} );
-}
-
 1;
 
 __END__
@@ -41,10 +40,10 @@ __XS__
 MODULE = KinoSearch    PACKAGE = KinoSearch::Search::TermScorer
 
 void
-_init_child(obj)
-    Scorer *obj;
+_init_child(scorer)
+    Scorer *scorer;
 PPCODE:
-    Kino_TermScorer_init_child(obj);
+    Kino_TermScorer_init_child(scorer);
 
 =for comment
 
@@ -54,85 +53,32 @@ be continually recalculated.
 =cut
 
 void
-_fill_score_cache(obj)
-    Scorer* obj;
-PREINIT:
-    TermScorerChild *child;
-    float           *cache_ptr;
-    int              i;
+_fill_score_cache(scorer)
+    Scorer* scorer;
 PPCODE:
-{
-    child = (TermScorerChild*)obj->child;
-    if (child->score_cache != NULL)
-        Kino_Safefree(child->score_cache);
-    Kino_New(0, child->score_cache, KINO_SCORE_CACHE_SIZE, float);
-
-    cache_ptr     = child->score_cache;
-    for (i = 0; i < KINO_SCORE_CACHE_SIZE; i++) {
-        *cache_ptr++ = obj->sim->tf(obj->sim, i) * child->weight_value;
-    }
-}
+    Kino_TermScorer_fill_score_cache(scorer);
 
 void
-_do_score_batch(obj, start, end, hc)
-    Scorer       *obj;
-    U32           start;
-    U32           end;
-    HitCollector *hc;
+score_batch(scorer, ...)
+    Scorer *scorer;
 PREINIT:
-    TermScorerChild *child;
-    U32              freq;
-    unsigned char    norm;
-    float            score;
-    int              i;
+    HV           *args_hash;
+    U32           start, end;
+    HitCollector *hc;
 PPCODE:
-{
-    obj->next(obj);
+    /* process hash-style params */
+    Kino_Verify_build_args_hash(args_hash, 
+        "KinoSearch::Search::TermScorer::score_batch_args", 1);
+    Kino_extract_struct_from_hv(args_hash, hc, "hit_collector", 13, 
+        HitCollector*, "KinoSearch::Search::HitCollector");
+    start = (U32)SvUV( Kino_Verify_extract_arg(args_hash, "start", 5) );
+    end   = (U32)SvUV( Kino_Verify_extract_arg(args_hash, "end", 3) );
 
-    child = (TermScorerChild*)obj->child;
-
-    while(child->doc < end) {
-        freq    = child->freqs[child->pointer];
-        if (freq < KINO_SCORE_CACHE_SIZE) {
-            /* cache hit, so we don't need to recompute the whole score */
-            score = child->score_cache[freq];
-        }
-        else {
-            score = obj->sim->tf(obj->sim, freq) * child->weight_value;
-        }
-
-        /* normalize for field */
-        norm = child->norms[child->doc];
-        score *= obj->sim->norm_decoder[norm];
-
-        hc->collect(hc, score, child->doc);
-        
-        /* time for a refill? */
-        if (++child->pointer >= child->pointer_max) {
-            /* try to get more docs and freqs */
-            child->pointer_max = child->term_docs->read(child->term_docs, 
-                child->doc_nums_sv, child->freqs_sv, 1024);
-            child->doc_nums = (U32*)SvPV_nolen(child->doc_nums_sv);
-            child->freqs    = (U32*)SvPV_nolen(child->freqs_sv);
-
-            /* bail if we didn't get any more docs */
-            if (child->pointer_max != 0) {
-                child->pointer = 0;
-            }
-            else {
-                child->doc = KINO_TERM_SCORER_SENTINEL;
-                /* TODO Lucene calls termDocs.close() here. */
-                XSRETURN(0);
-            }
-        }
-
-        child->doc = child->doc_nums[ child->pointer ];
-    }
-}
+    Kino_TermScorer_score_batch(scorer, start, end, hc);
 
 SV*
-_term_scorer_set_or_get(obj, ...)
-    Scorer *obj;
+_term_scorer_set_or_get(scorer, ...)
+    Scorer *scorer;
 ALIAS:
     _set_term_docs    = 1
     _get_term_docs    = 2
@@ -146,15 +92,14 @@ PREINIT:
     TermScorerChild *child;
 CODE:
 {
-    child = (TermScorerChild*)obj->child;
+    child = (TermScorerChild*)scorer->child;
     /* if called as a setter, make sure the extra arg is there */
     if (ix % 2 == 1 && items != 2)
         croak("usage: $scorer->set_xxxxxx($val)");
 
     switch (ix) {
 
-    case 1:  if (child->term_docs_sv != NULL)
-                SvREFCNT_dec(child->term_docs_sv);
+    case 1:  SvREFCNT_dec(child->term_docs_sv);
              child->term_docs_sv = newSVsv( ST(1) );
              Kino_extract_struct( child->term_docs_sv, child->term_docs, 
                 TermDocs*, "KinoSearch::Index::TermDocs");
@@ -162,10 +107,9 @@ CODE:
     case 2:  RETVAL = newSVsv(child->term_docs_sv);
              break;
 
-    case 3:  if (!sv_derived_from( ST(1), "KinoSearch::Search::Weight"))
+    case 3:  SvREFCNT_dec(child->weight_sv); 
+             if (!sv_derived_from( ST(1), "KinoSearch::Search::Weight"))
                 Kino_confess("not a KinoSearch::Search::Weight");
-             if (child->weight_sv != NULL)
-                SvREFCNT_dec(child->weight_sv);
              child->weight_sv = newSVsv( ST(1) );
              /* fall through */
     case 4:  RETVAL = newSVsv(child->weight_sv);
@@ -176,8 +120,7 @@ CODE:
     case 6:  RETVAL = newSVnv(child->weight_value);
              break;
 
-    case 7:  if (child->norms_sv != NULL) 
-                SvREFCNT_dec(child->norms_sv);
+    case 7:  SvREFCNT_dec(child->norms_sv);
              child->norms_sv = newSVsv( ST(1) );
              {
                  SV* bytes_deref_sv;
@@ -197,10 +140,10 @@ CODE:
 OUTPUT: RETVAL
 
 void
-DESTROY(obj)
-    Scorer *obj;
+DESTROY(scorer)
+    Scorer *scorer;
 PPCODE:
-    Kino_TermScorer_destroy(obj);
+    Kino_TermScorer_destroy(scorer);
 
 __H__
 
@@ -213,6 +156,7 @@ __H__
 #include "EXTERN.h"
 #include "perl.h"
 #include "KinoSearchIndexTermDocs.h"
+#include "KinoSearchSearchHitCollector.h"
 #include "KinoSearchSearchScorer.h"
 #include "KinoSearchUtilMemManager.h"
 
@@ -234,10 +178,12 @@ typedef struct termscorerchild {
 } TermScorerChild;
 
 void Kino_TermScorer_init_child(Scorer*);
-void Kino_TermScorer_destroy(Scorer*);
+void Kino_TermScorer_fill_score_cache(Scorer*);
 bool Kino_TermScorer_next(Scorer*);
 float Kino_TermScorer_score(Scorer*);
+void  Kino_TermScorer_score_batch(Scorer*, U32, U32, HitCollector*);
 U32 Kino_TermScorer_doc(Scorer*);
+void Kino_TermScorer_destroy(Scorer*);
 
 #endif /* include guard */
 
@@ -270,25 +216,37 @@ Kino_TermScorer_init_child(Scorer *scorer){
     child->weight_value = 0.0;
     child->norms        = NULL;
     child->score_cache  = NULL;
-    child->weight_sv    = NULL;
-    child->term_docs_sv = NULL;
-    child->norms_sv     = NULL;
+    child->weight_sv    = &PL_sv_undef;
+    child->term_docs_sv = &PL_sv_undef;
+    child->norms_sv     = &PL_sv_undef;
 }   
 
+void
+Kino_TermScorer_fill_score_cache(Scorer *scorer) {
+    TermScorerChild *child;
+    float           *cache_ptr;
+    int              i;
+
+    child = (TermScorerChild*)scorer->child;
+    Kino_Safefree(child->score_cache);
+    Kino_New(0, child->score_cache, KINO_SCORE_CACHE_SIZE, float);
+
+    cache_ptr     = child->score_cache;
+    for (i = 0; i < KINO_SCORE_CACHE_SIZE; i++) {
+        *cache_ptr++ = scorer->sim->tf(scorer->sim, i) * child->weight_value;
+    }
+}
+
 void
 Kino_TermScorer_destroy(Scorer *scorer) {
     TermScorerChild *child;
     child = (TermScorerChild*)scorer->child;
 
-    if (child->score_cache != NULL)
-        Kino_Safefree(child->score_cache);
+    Kino_Safefree(child->score_cache);
 
-    if (child->term_docs_sv != NULL) 
-        SvREFCNT_dec(child->term_docs_sv);
-    if (child->norms_sv != NULL) 
-        SvREFCNT_dec(child->norms_sv);
-    if (child->weight_sv != NULL) 
-        SvREFCNT_dec(child->weight_sv);
+    SvREFCNT_dec(child->term_docs_sv);
+    SvREFCNT_dec(child->norms_sv);
+    SvREFCNT_dec(child->weight_sv);
     SvREFCNT_dec(child->doc_nums_sv);
     SvREFCNT_dec(child->freqs_sv);
 
@@ -345,6 +303,58 @@ Kino_TermScorer_score(Scorer* scorer) {
     return score;
 }
 
+void
+Kino_TermScorer_score_batch(Scorer *scorer, U32 start, U32 end,
+                               HitCollector* hc) {
+    TermScorerChild *child;
+    U32              freq;
+    unsigned char    norm;
+    float            score;
+    int              i;
+
+    child = (TermScorerChild*)scorer->child;
+
+    scorer->next(scorer);
+
+    while(child->doc < end) {
+        freq = child->freqs[child->pointer];
+        if (freq < KINO_SCORE_CACHE_SIZE) {
+            /* cache hit, so we don't need to recompute the whole score */
+            score = child->score_cache[freq];
+        }
+        else {
+            score = scorer->sim->tf(scorer->sim, freq) * child->weight_value;
+        }
+
+        /* normalize for field */
+        norm = child->norms[child->doc];
+        score *= scorer->sim->norm_decoder[norm];
+
+        hc->collect(hc, child->doc, score);
+        
+        /* time for a refill? */
+        if (++child->pointer >= child->pointer_max) {
+            /* try to get more docs and freqs */
+            child->pointer_max = child->term_docs->read(child->term_docs, 
+                child->doc_nums_sv, child->freqs_sv, 1024);
+            child->doc_nums = (U32*)SvPV_nolen(child->doc_nums_sv);
+            child->freqs    = (U32*)SvPV_nolen(child->freqs_sv);
+
+            /* bail if we didn't get any more docs */
+            if (child->pointer_max != 0) {
+                child->pointer = 0;
+            }
+            else {
+                child->doc = KINO_TERM_SCORER_SENTINEL;
+                /* TODO Lucene calls termDocs.close() here. */
+                return;
+            }
+        }
+
+        child->doc = child->doc_nums[ child->pointer ];
+    }
+}
+
 U32 
 Kino_TermScorer_doc(Scorer* scorer) {
     TermScorerChild *child = (TermScorerChild*)scorer->child;
@@ -370,7 +380,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
