@@ -1,4 +1,6 @@
 package KinoSearch::Util::Class;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 
 use Clone 'clone';
@@ -243,7 +245,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =cut
 
