@@ -1,4 +1,6 @@
 package KinoSearch::Util::ToStringUtils;
+use strict;
+use warnings;
 use KinoSearch::Util::ToolSet;
 use base qw( Exporter );
 
@@ -30,7 +32,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.05_05.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
